/************************************************|
|                                                |
|          Procedure ETL_SDXMAPGESTAO            |
|                                                |
|                                                |
|      Rotina para Insero de Dados de BI-      |
|              Mapa de Gestao SDX                |
|                                                |
|               Tabela Preenchida:               |
|                                                |
|                  FATORESUFAT                   |
|                                                |
|     Criao:                                   |
|     15/05/2008 - Leandro Corra Gonalves      |
|                                                |
*************************************************/

CREATE OR REPLACE PROCEDURE ETL_SDXMAPGESTAO
                     (P_DTINI      IN DATE,
                      P_DTFINA     IN DATE,
                      P_CDGRUPPROD IN VARCHAR)

IS
  V_DTFIM           DATE;
  V_DTINI           DATE;
  V_NRSEQFATRES     FATORESUFAT.NRSEQFATRES%TYPE;
  V_DESPESA         NOTAFISCAL.VRNOTAFISC%TYPE;
  V_MOBRA           NOTAFISCAL.VRNOTAFISC%TYPE;
  V_VRFATUR         NOTAFISCAL.VRNOTAFISC%TYPE;
  V_VRIMPOSTO       NOTAFISCAL.VRNOTAFISC%TYPE;
  V_FATLIQ          NOTAFISCAL.VRNOTAFISC%TYPE;
  V_ENTRADANF_LIQ   NOTAFISCAL.VRNOTAFISC%TYPE;
  V_ENTRADANF       NOTAFISCAL.VRNOTAFISC%TYPE;
  V_ESTOQUE_INICIAL NOTAFISCAL.VRNOTAFISC%TYPE;
  V_ESTOQUE_FINAL   NOTAFISCAL.VRNOTAFISC%TYPE;
  V_SAIDANF_LIQ     NOTAFISCAL.VRNOTAFISC%TYPE;
  V_SAIDANF         NOTAFISCAL.VRNOTAFISC%TYPE;
  V_CONSUMO         NOTAFISCAL.VRNOTAFISC%TYPE;
  V_RESULT          NOTAFISCAL.VRNOTAFISC%TYPE;
  V_CDGRUPPROD      GRUPPROD.CDGRUPPROD%TYPE;
  V_DTMES           VARCHAR2(02);
  V_DTANO           VARCHAR2(04);
  V_TIPOFATU        VARCHAR2(30);
  V_COUNT           NUMBER := 0;
  V_ZERO            NUMBER := 0;
  V_CUSTOS          FLOAT(126);
  V_CUSTAJUS        FLOAT(126);


BEGIN

    BEGIN
       SELECT NVL(MAX(NRSEQFATRES),0)
         INTO   V_NRSEQFATRES
         FROM FATORESUFAT;
    EXCEPTION
    WHEN OTHERS THEN
         V_NRSEQFATRES := '0';
    END;

    V_DTMES  :=  TO_CHAR(P_DTINI,'MM');
    V_DTANO  :=  TO_CHAR(P_DTINI,'YYYY');
    V_CDGRUPPROD := P_CDGRUPPROD;

    DELETE FROM FATORESUFAT
    WHERE  (ANFATO > V_DTANO) OR
           (ANFATO = V_DTANO AND MEFATO >= V_DTMES);
    COMMIT;

    V_DTFIM  := LAST_DAY(V_DTINI);
    V_DTINI  := P_DTINI;

    WHILE (V_DTINI  <=  P_DTFINA) LOOP

          V_DTFIM := LAST_DAY( V_DTINI);

          FOR C_FILIAL IN
             (SELECT F.CDFILIAL, CDEMPRESA,
                     F.CDAREAGERE      AS CDAREAGERE,
                     F.CDGOPERAC       AS CDGOPERAC,
                     F.CDSUPERVISOR  AS CDSUPERVISOR,
                     F.CDSEGMER          AS CDSEGMER,
                     P.CDTIPOFATUR     AS CDTIPOFATUR,
                     NVL(F.QTFUNCFILI,1)     AS QTFUNCFILI,
                     P.DTPROCESSA
              FROM   FILIAL F,  PARAMFILIAL P
              WHERE  (F.CDFILIAL = P.CDFILIAL)
              )
          LOOP

             /* Faturamento bruto  */

             BEGIN
               SELECT SUM(DECODE(IDTPLANCROP,'FS',NVL(VRITEMLANCROP,0),0)) VRTOTFATSAID
                 INTO V_VRFATUR
                 FROM LANCROPFILIAL L, ITLANCROPFILIAL IT
                WHERE (L.CDFILIAL     = C_FILIAL.CDFILIAL)
                  AND (L.IDTPLANCROP  IN('DD','DM','FS'))
                  AND (L.DTLANCROP  BETWEEN V_DTINI AND V_DTFIM)
                  AND (L.CDFILIAL      = IT.CDFILIAL)
                  AND (L.NRROPFILIAL   = IT.NRROPFILIAL)
                  AND (L.NRLANCROP     = IT.NRLANCROP);
             EXCEPTION
             WHEN OTHERS THEN
                  V_VRFATUR   := V_ZERO;
             END;

             /* IMPOSTOS  */

             BEGIN
               SELECT SUM(NVL(A.VRIMPOSTO,0)) AS IMPOSTO
                 INTO V_VRIMPOSTO
                 FROM (SELECT IT.CDFILIAL,CDITRELITROP, DTLANCROP, SGIMPOSTO,
                              SUM(NVL(VRITRELITROP,0)) VRIMPOSTO
                         FROM ITRELACITROP IR,LANCROPFILIAL L,
                              ITLANCROPFILIAL IT, IMPOSTO IP
                         WHERE (L.CDFILIAL      = C_FILIAL.CDFILIAL)
                           AND (L.DTLANCROP BETWEEN V_DTINI AND V_DTFIM)
                           AND (L.CDFILIAL      = IT.CDFILIAL)
                           AND (L.NRROPFILIAL   = IT.NRROPFILIAL)
                           AND (L.NRLANCROP     = IT.NRLANCROP)
                           AND (IT.CDFILIAL     = IR.CDFILIAL(+))
                           AND (IT.NRROPFILIAL  = IR.NRROPFILIAL(+))
                           AND (IT.NRLANCROP    = IR.NRLANCROP(+))
                           AND (IT.NRITLANCROP  = IR.NRITLANCROP(+))
                           AND (L.IDTPLANCROP   = 'FS')
                           AND (IR.CDITRELITROP = IP.CDIMPOSTO)
                         GROUP BY IT.CDFILIAL,CDITRELITROP, DTLANCROP, SGIMPOSTO
                         ORDER BY DTLANCROP, CDITRELITROP) A
               GROUP BY A.CDFILIAL;
             EXCEPTION
             WHEN OTHERS THEN
                  V_VRIMPOSTO := V_ZERO;
             END;

             /* Mao Obra */

             BEGIN
               SELECT NVL(SUM(A.VRREALNESTO),0)
                 INTO V_MOBRA
                 FROM CUSTNESTO A,PRODUTO P
                 WHERE (A.CDFILIAL = C_FILIAL.CDFILIAL)
                   AND (A.DTCUSTNESTO BETWEEN V_DTINI AND V_DTFIM)
                   AND (A.CDPRODUTO = P.CDPRODUTO)
                   AND ((SUBSTR(CDARVPROD,1,1) <> 'D') AND (SUBSTR(CDARVPROD,1,1) <> 'P')
                   AND (SUBSTR(CDARVPROD,1,1) <> 'F') AND (SUBSTR(CDARVPROD,1,1) <> 'O')) ;
             EXCEPTION
             WHEN OTHERS THEN
                  V_MOBRA := V_ZERO;
             END;

             /* Despesas  */

             BEGIN
               SELECT SUM(VRTOTDESP)
                 INTO V_DESPESA
                 FROM (SELECT SUM(VALOR) AS VRTOTDESP, DESP.CDFILIAL
                        FROM
                            (SELECT SUM(((NVL(I.VRTOTAITEM,0) - NVL(I.VRDESCITEM,0)) - ((NVL(I.VRTOTAITEM,0) / NVL(N.VRPRODUTOS,0))
                                    * NVL(T.VRITEMENCA,0)))) VALOR, N.CDFILIAL
                               FROM NOTAFISCAL N, ITEMPRODUTO I, PRODUTO P, ITEMENCARGO T
                              WHERE (N.IDENTRSAIDOP  = 'E')
                                AND (N.CDFILIRECE    = C_FILIAL.CDFILIAL)
                                AND (N.CDTIPOOPER IN ('01','10'))
                                AND (N.DTENTRSAID BETWEEN V_DTINI AND V_DTFIM)
                                AND (N.IDSTATUS  <> 'C')
                                AND (N.CDFILIAL = I.CDFILIAL)
                                AND (N.NRLANCTONF = I.NRLANCTONF)
                                AND (N.CDFILIAL     = T.CDFILIAL(+))
                                AND (N.NRLANCTONF   = T.NRLANCTONF(+))
                                AND (T.CDENCARGO(+) = '03')
                                AND ((P.CDARVPROD LIKE 'D%') OR (P.CDARVPROD LIKE 'P%'))
                                AND (I.CDPRODUTO = P.CDPRODUTO)
                              GROUP BY N.CDFILIAL

                             UNION ALL

                             SELECT NVL(SUM(A.VRREALNESTO),0) AS VALOR, A.CDFILIAL
                               FROM CUSTNESTO A,PRODUTO P
                              WHERE (A.CDFILIAL   = C_FILIAL.CDFILIAL)
                                AND (A.DTCUSTNESTO BETWEEN V_DTINI AND V_DTFIM)
                                AND (A.CDPRODUTO  = P.CDPRODUTO)
                                AND ((P.CDARVPROD LIKE 'D%') OR (P.CDARVPROD LIKE 'P%'))
                              GROUP BY A.CDFILIAL)DESP
                        GROUP BY DESP.CDFILIAL

                       UNION ALL

                       SELECT NVL(SUM(A.VRREALNESTO),0) VALOR, A.CDFILIAL
                         FROM CUSTNESTO A, PRODUTO P
                        WHERE (A.CDFILIAL = C_FILIAL.CDFILIAL)
                          AND (A.DTCUSTNESTO BETWEEN V_DTINI AND V_DTFIM)
                          AND (A.CDPRODUTO = P.CDPRODUTO)
                          AND ((SUBSTR(CDARVPROD,1,1) <> 'D') AND (SUBSTR(CDARVPROD,1,1) <> 'P')
                          AND (SUBSTR(CDARVPROD,1,1) <> 'F') AND (SUBSTR(CDARVPROD,1,1) <> 'O'))
                        GROUP BY CDARVPROD, P.NMPRODUTO,A.CDFILIAL) DESPESA

                GROUP BY DESPESA.CDFILIAL;
               EXCEPTION
               WHEN OTHERS THEN
                    V_DESPESA := V_ZERO;
               END;


             /* ESTOQUE INICIAL, ESTOQUE FINAL E VALOR DE ENTRADA */

             BEGIN
               SELECT SUM(NVL(VRTOTINVINIC,0)) , SUM(NVL(VRTOTINVFINAL,0)), SUM(NVL(VRTOTENTRADA,0)), SUM(NVL(VRTOTAJUSTE,0))
                 INTO V_ESTOQUE_INICIAL, V_ESTOQUE_FINAL,V_ENTRADANF_LIQ, V_ENTRADANF
                 FROM ( SELECT CDFILIAL,CDSUBGRPROD,
                               SUM(NVL(VRTOTINVINIC,0)) VRTOTINVINIC,
                               SUM(NVL(VRTOTINVFINAL,0)) VRTOTINVFINAL,
                               SUM(NVL(VRTOTENTRADA,0)) VRTOTENTRADA,
                               SUM(NVL(VRTOTAJUSTE,0)) VRTOTAJUSTE
                          FROM ( SELECT GR.CDGRUPPROD,  SG.CDSUBGRPROD, L.CDFILIAL,
                                        (0) VRTOTINVINIC, (0) VRTOTINVFINAL,
                                        DECODE(IDTPLANCROP,'CC',NVL(VRITEMLANCROP,0),0) VRTOTENTRADA,
                                        DECODE(IDTPLANCROP,'AJ',NVL(VRITEMLANCROP,0),0) VRTOTAJUSTE
                                   FROM
                                       LANCROPFILIAL L, GRUPPROD GR, SUBGRUPPROD SG,
                                       PRSUBGRPROD AN,  ITLANCROPFILIAL IT
                                  WHERE (L.CDFILIAL     = C_FILIAL.CDFILIAL)
                                    AND (L.IDTPLANCROP IN ('CC','IM','AJ'))
                                    AND (L.DTLANCROP  BETWEEN V_DTINI AND V_DTFIM)
                                    AND (L.CDFILIAL      = IT.CDFILIAL)
                                    AND (L.NRROPFILIAL   = IT.NRROPFILIAL)
                                    AND (L.NRLANCROP     = IT.NRLANCROP)
                                    AND (GR.CDGRUPPROD   = V_CDGRUPPROD)
                                    AND (GR.CDGRUPPROD   = SG.CDGRUPPROD)
                                    AND (SG.CDSUBGRPROD   = AN.CDSUBGRPROD)
                                    AND (IT.CDITEMLANCROP = AN.CDPRODUTO)
                                    AND (SG.CDGRUPPROD    = AN.CDGRUPPROD)

                                 UNION ALL

                                 SELECT GR.CDGRUPPROD,SG.CDSUBGRPROD,L.CDFILIAL,
                                        DECODE(IDTPLANCROP,'II',NVL(VRITEMLANCROP,0),0) VRTOTINVINIC,
                                        DECODE(IDTPLANCROP,'IF',NVL(VRITEMLANCROP,0),0) VRTOTINVFINAL,
                                        (0) VRTOTENTRADA, (0) VRTOTAJUSTE
                                   FROM
                                       LANCROPFILIAL L, GRUPPROD GR,
                                       SUBGRUPPROD SG,    ITLANCROPFILIAL IT
                                   WHERE (L.CDFILIAL     = C_FILIAL.CDFILIAL)
                                     AND ((L.DTLANCROP  = V_DTINI AND IDTPLANCROP = 'II') OR
                                          (L.DTLANCROP = V_DTFIM AND IDTPLANCROP = 'IF'))
                                     AND (L.CDFILIAL      = IT.CDFILIAL)
                                     AND (L.NRROPFILIAL   = IT.NRROPFILIAL)
                                     AND (L.NRLANCROP     = IT.NRLANCROP)
                                     AND (GR.CDGRUPPROD   = V_CDGRUPPROD)
                                     AND (GR.CDGRUPPROD   = SG.CDGRUPPROD)
                                     AND (IT.CDITEMLANCROP = SG.CDSUBGRPROD))
                                   GROUP BY CDGRUPPROD,  CDSUBGRPROD, CDFILIAL
                                   ORDER BY CDSUBGRPROD)A
               GROUP BY A.CDFILIAL ;
             EXCEPTION
             WHEN OTHERS THEN
                 V_ESTOQUE_INICIAL := V_ZERO;
                 V_ESTOQUE_FINAL   := V_ZERO;
                 V_ENTRADANF_LIQ   := V_ZERO;
                 V_ENTRADANF       := V_ZERO;
             END;

             /* VALOR DE SAIDA */

             BEGIN
               SELECT SUM(NVL(VRTOTCONSSAID,0)) AS VRSAIDA
                 INTO V_SAIDANF
                 FROM (SELECT GR.CDGRUPPROD,  SG.CDSUBGRPROD,
                              SUM(DECODE(IDTPLANCROP,'CS',NVL(VRITEMLANCROP,0),0)) VRTOTCONSSAID
                         FROM
                            LANCROPFILIAL L, GRUPPROD GR, SUBGRUPPROD SG,
                            PRSUBGRPROD AN,  ITLANCROPFILIAL IT
                         WHERE (L.CDFILIAL      = C_FILIAL.CDFILIAL)                  
                           AND (L.IDTPLANCROP   = 'CS')                            
                           AND (L.DTLANCROP  BETWEEN V_DTINI AND V_DTFIM)          
                           AND (L.CDFILIAL       = IT.CDFILIAL)                   
                           AND (L.NRROPFILIAL    = IT.NRROPFILIAL)                 
                           AND (L.NRLANCROP      = IT.NRLANCROP)                   
                           AND (GR.CDGRUPPROD    = V_CDGRUPPROD)                   
                           AND (GR.CDGRUPPROD    = SG.CDGRUPPROD)                  
                           AND (SG.CDSUBGRPROD   = AN.CDSUBGRPROD)                 
                           AND (IT.CDITEMLANCROP = AN.CDPRODUTO)                   
                           AND (SG.CDGRUPPROD    = AN.CDGRUPPROD)
                         GROUP BY GR.CDGRUPPROD,  SG.CDSUBGRPROD
                         ORDER BY SG.CDSUBGRPROD)A
               GROUP BY A.CDGRUPPROD;
             EXCEPTION
             WHEN OTHERS THEN
                  V_SAIDANF := V_ZERO;
             END;
             
             /* VALOR DO CUSTO */ 
             
             BEGIN  
               SELECT SUM(NVL(VRREALNESTO,0)) VRCUSTO
                 INTO V_CUSTOS
                 FROM CUSTNESTO A
                WHERE (A.CDFILIAL = C_FILIAL.CDFILIAL)                                                                      
                  AND (A.DTCUSTNESTO BETWEEN V_DTINI AND V_DTFIM)  
                  AND ((SUBSTR(A.CDPRODUTO,1,1) IN ('F','O')));
            EXCEPTION
            WHEN OTHERS THEN
                 V_CUSTOS := V_ZERO;
            END;
            
            V_CONSUMO := (V_ESTOQUE_INICIAL + V_CUSTOS + (V_ENTRADANF_LIQ - V_SAIDANF) -
                           V_ESTOQUE_FINAL);

            V_FATLIQ  :=  V_VRFATUR + V_VRIMPOSTO; -- Soma pois o imposto ja vem negativo

            V_RESULT  :=  (V_FATLIQ - V_DESPESA - V_MOBRA - V_CONSUMO );

            V_DTMES  :=  TO_CHAR(V_DTFIM,'MM');
            V_DTANO  :=  TO_CHAR(V_DTFIM,'YYYY');

            V_TIPOFATU :=
                CASE  C_FILIAL.CDTIPOFATUR
                   WHEN '001'  THEN   'Gesto Direta'
                   WHEN '003'  THEN   'Mandato'
                   WHEN '004'  THEN   'Refeies e Servicos'
                   WHEN '005'  THEN   'Taxa Fixa'
                   WHEN '006'  THEN   'Gesto Hospitalar'
                   WHEN '007'  THEN   'Produtos e Servios'
                   WHEN '008'  THEN   'Produtos e Servios'
                ELSE 'Sem Tipo Faturamento'
                END;

             IF  V_FATLIQ = 0 THEN
                V_FATLIQ := 1;
             END IF;

             IF  C_FILIAL.QTFUNCFILI = 0 THEN
                C_FILIAL.QTFUNCFILI := 1;
             END IF;

             IF  V_VRFATUR > 0 OR V_DESPESA > 0 OR V_MOBRA >0 OR V_CONSUMO > 0 THEN
                V_NRSEQFATRES := V_NRSEQFATRES + 1;

                INSERT INTO FATORESUFAT
                   (CDEMPRESA,  CDFILIAL,  CDAREAGERE ,  CDGOPERAC,
                    CDSUPERVISOR, CDSEGMER,  CDTIPOFATURE, CDSERVFILI ,
                    ANFATO,  MEFATO , VRFATOFATU, VRFATOIMPO,
                    QTFATOCOME, QTFATOCOMF,  VRFATOCONS, VRMAOOBRA,
                    VRFATOMATPR, VRFATODESPE, VRFATOFUNC, VRFATOLIQNF,
                    VRFATORESU, QTFATOFUNC, VRFATOMARG, VRPERMDOFAT,
                    VRCUSTOMDO, NRSEQFATRES, DSDTPROCFATOFAT)
                VALUES
                   (C_FILIAL.CDEMPRESA, C_FILIAL.CDFILIAL, C_FILIAL.CDAREAGERE, C_FILIAL.CDGOPERAC,
                    C_FILIAL.CDSUPERVISOR, C_FILIAL.CDSEGMER, V_TIPOFATU, NULL,
                    V_DTANO, V_DTMES, V_VRFATUR, V_VRIMPOSTO,
                    0, 0, V_CONSUMO, V_MOBRA,
                    0, V_DESPESA, (V_FATLIQ / C_FILIAL.QTFUNCFILI), V_FATLIQ,
                    V_RESULT, C_FILIAL.QTFUNCFILI, ((V_RESULT / V_FATLIQ) *100), ((V_MOBRA / V_FATLIQ) *100),
                    (V_MOBRA / C_FILIAL.QTFUNCFILI), LPAD(V_NRSEQFATRES,12,0),TO_CHAR(C_FILIAL.DTPROCESSA,'DD/MM/YYYY'));   

             END IF;

             V_COUNT := V_COUNT + 1;
             IF V_COUNT >= 5 THEN
                COMMIT;
                V_COUNT := 0;
             END IF;
          END LOOP;

          COMMIT;
          V_DTINI := ADD_MONTHS(V_DTINI,1);
    END LOOP;
END;
/


/*REGISTRANDO DIMENSES*/

BEGIN                                

   CRIA_DIME('FATORESUFAT',  'ANFATO',       'Ano',                  NULL,           NULL,            NULL,          NULL); 
   CRIA_DIME('FATORESUFAT',  'MEFATO',       'Ms',                  NULL,           NULL,            NULL,          NULL);     
   CRIA_DIME('FATORESUFAT',  'CDFILIAL',     'Filial',              'FILIAL',       'CDFILIAL',      'NMFILIAL',     NULL); 
   CRIA_DIME('FATORESUFAT',  'CDEMPRESA',    'Empresa',             'EMPRESA',      'CDEMPRESA',     'NMEMPRESA',    NULL); 
   CRIA_DIME('FATORESUFAT',  'CDAREAGERE',   'Regional',            'AREAGERE',     'CDAREAGERE',    'NMAREAGERE',   NULL); 
   CRIA_DIME('FATORESUFAT',  'CDFORNECED',   'Fornecedor',          'FORNECEDOR',   'CDFORNECED',    'NMFANTFORN',   NULL);    
   CRIA_DIME('FATORESUFAT',  'CDGOPERAC',    'Gerente Operacional', 'GOPERAC',      'CDGOPERAC',     'NMGOPERAC',    NULL);
   CRIA_DIME('FATORESUFAT',  'CDSUPERVISOR', 'Supervisor',          'SUPERVISOR',   'CDSUPERVISOR',  'NMSUPERVISOR', NULL);          
   CRIA_DIME('FATORESUFAT',  'CDSERVFILI',   'Servio',             'FILISERVFILI', 'CDFILISERVFILI','NMSERVICO',    NULL);
   CRIA_DIME('FATORESUFAT',  'CDTIPOFATURE', 'Tipo de Faturamento',  NULL,           NULL,            NULL,          NULL);
   CRIA_DIME('FATORESUFAT',  'CDSEGMER',     'Segmento de Mercado', 'SEGMERCADO',   'CDEMPRESA',     'NMEMPRESA',    NULL);
   
   COMMIT; 
END;
/
       

DECLARE
   V_CDSEQUENCIAL VARCHAR2(20) NULL;
BEGIN
    /*REGISTRANDO A PROCEDURE PARA O AGENDADOR DE EVENTOS*/
    SELECT COUNT(*)
    INTO   V_CDSEQUENCIAL
    FROM   EVENTOAEV 
    WHERE  NMOBJBDEVEN = 'ETL_SDXMAPGESTAO';
    
    IF V_CDSEQUENCIAL = 0 THEN     --> S ADICIONA A PROCEDURE SE ELA NO EXISTIR AINDA NO CADASTRO
        BEGIN
        
            SELECT LPAD(NVL(MAX(LPAD(CDEVENTOAEV, 10, 0))+1,0),10,'0')
            INTO   V_CDSEQUENCIAL
            FROM   EVENTOAEV;
        
            INSERT INTO EVENTOAEV 
            (CDEVENTOAEV, NMEVENTOAEV, IDTPEVENAEV, CDOPERINCL, DTINCLEVEN, NMOBJBDEVEN)
            VALUES
            (V_CDSEQUENCIAL,'ETL Mapa de Gesto','STP', NULL, SYSDATE, 'ETL_SDXMAPGESTAO');
            COMMIT;
            
        EXCEPTION
           WHEN OTHERS THEN
              RAISE_APPLICATION_ERROR (-20001, 'Falha no cadastro de eventos agendveis.');
        END;
    END IF;
END;
/


-- VERSAO --
BEGIN ATUVEROBJ('ETL_FATORESUF','5.03.009'); END;
/
